use strict;
use Test::More tests => 35;
use DE_EPAGES::Catalog::API::Catalog qw( AssignProductToCategory IsProductAssignedToCategory IsProductAssignedToCategoryWithParent RemoveProductFromCategory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Category = LoadObjectByPath('/Test/TestShop/Categories/Shirts');
    my $Product = LoadObjectByPath('/Test/TestShop/Products/Shirt');
    my $SubProduct = LoadObjectByPath('/Test/TestShop/Products/Shirt/SubProducts/Shirt blue');
    my $CategorySpecial = LoadObjectByPath('/Test/TestShop/Categories/SpecialOffers');

    ok (!$Product->get('IsSpecialOffer'), 'product is not special offer, missing category assignment');
    ok (!$SubProduct->get('IsSpecialOffer'), 'subproduct is not special offer, missing category assignment');

    ok (!IsProductAssignedToCategory($Product, $Category), 'product is not assigned to category');
    ok (!IsProductAssignedToCategory($SubProduct, $Category), 'subproduct is not assigned to category');

    ok (AssignProductToCategory($Product, $Category) == 1, 'product has been assigned');
    ok (!$Product->get('IsSpecialOffer'), 'product is not special offer, missing special assignment');
    ok (AssignProductToCategory($Product, $Category) == 2, 'product already assigned before');

    ok (IsProductAssignedToCategory($Product, $Category), 'product is assigned to category');
    ok (!IsProductAssignedToCategory($SubProduct, $Category), 'subproduct is not assigned to category');
    ok (IsProductAssignedToCategoryWithParent($SubProduct, $Category), 'parent or subproduct are assigned to category');

    ok (AssignProductToCategory($SubProduct, $Category) == 3, 'subproduct has not been assigned - super product already assigned before');

    ok (RemoveProductFromCategory($Product, $Category) == 1, 'product has been removed');
    ok (RemoveProductFromCategory($Product, $Category) == 2, 'product had not been already assigned before');

    ok (!IsProductAssignedToCategory($Product, $Category), 'product is not assigned to category');
    ok (!IsProductAssignedToCategory($SubProduct, $Category), 'subproduct is not assigned to category');
    ok (!IsProductAssignedToCategoryWithParent($SubProduct, $Category), 'parent and subproduct are not assigned to category');

    ok (AssignProductToCategory($SubProduct, $Category) == 1, 'subproduct has been assigned');
    ok (!$SubProduct->get('IsSpecialOffer'), 'subproduct is not special offer, missing special assignment');

    ok (!IsProductAssignedToCategory($Product, $Category), 'product is not assigned to category');
    ok (IsProductAssignedToCategory($SubProduct, $Category), 'subproduct is assigned to category');
    ok (IsProductAssignedToCategoryWithParent($SubProduct, $Category), 'parent or subproduct are assigned to category');

    ok (RemoveProductFromCategory($SubProduct, $Category) == 1, 'subproduct has been removed');

    ok (AssignProductToCategory($Product, $CategorySpecial) == 1, 'product has been assigned as special offer');
    ok ($Product->get('IsSpecialOffer'), 'product is special offer');
    ok ($SubProduct->get('IsSpecialOffer'), 'subproduct is special offer');

    ok (RemoveProductFromCategory($Product, $CategorySpecial) == 1, 'product has been removed from special offer');
    ok (!$Product->get('IsSpecialOffer'), 'product is not special offer, missing special assignment');
    ok (!$SubProduct->get('IsSpecialOffer'), 'subproduct is not special offer, missing special assignment');

    ok (AssignProductToCategory($SubProduct, $CategorySpecial) == 1, 'subproduct has been assigned as special offer');
    ok (!$Product->get('IsSpecialOffer'), 'product is not special offer, missing special assignment');
    ok ($SubProduct->get('IsSpecialOffer'), 'subproduct is special offer');

    ok (RemoveProductFromCategory($SubProduct, $CategorySpecial) == 1, 'subproduct has been final removed from special offer');
    ok (!$SubProduct->get('IsSpecialOffer'), 'subproduct is not special offer, missing special assignment');

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
