use Test::More tests => 176;
use strict;

use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Domain qw (Urls PoolDBEntries TestWebServerScriptNamePart);
use DE_EPAGES::Object::API::Constants qw( ISWEBSERVERSCRIPTNAMEPARTUNIQUE EXPANDWWWDOMAIN );
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );

my $hSystem = {
    SiteID                  => 1, # identify system
    DomainName              => 'www.epages.de',
    HasSSLCertificate       => 1,
    WebServerScriptNamePart => 'Store',
    WebServerDirectory      => 'epages',
    URLStorefrontExtension  => 'sf',
    URLAdminExtension       => 'admin',
    URLPreviewExtension     => 'preview',
    WebServerPort           => 40,
};
my $hSystem2 = {
    SiteID                  => 1, # identify system
    DomainName              => 'www.epages.de',
    HasSSLCertificate       => 0,
    WebServerScriptNamePart => 'Store',
    WebServerDirectory      => 'epages',
    URLStorefrontExtension  => 'sf',
    URLAdminExtension       => 'admin',
    URLPreviewExtension     => 'preview',
    WebServerPort           => 40,
};
my $hSite1 = {
    SiteID                  => 2,
    DomainName              => 'www.myshop.de',
    HasSSLCertificate       => 0,
    WebServerScriptNamePart => 'shop',
};

my $hSite2 = {
    SiteID                  => 3,
    DomainName              => undef,
    HasSSLCertificate       => 0,
    WebServerScriptNamePart => 'myshop',
};

my $hSite3 = {
    SiteID                  => 4,
    DomainName              => '1.2.3.4',
    HasSSLCertificate       => 1,
    WebServerScriptNamePart => 'shop',
};

my $hSite4 = {
    SiteID                  => 5,
    DomainName              => '2001:4bd8:2:2:213:d3ff:fe62:f1f0',
    HasSSLCertificate       => 1,
    WebServerScriptNamePart => 'shop',
};

my $hSite5 = {
    SiteID                  => 6,
    DomainName              => undef,
    HasSSLCertificate       => 0,
    WebServerScriptNamePart => undef,
};

my $hSite6 = {
    SiteID                  => 7,
    DomainName              => 'myshop.de',
    SecondaryDomains        => ['yourshop.de'],
    HasSSLCertificate       => 0,
    WebServerScriptNamePart => 'shop',
};

my $hSite7 = {
    SiteID                  => 8,
    DomainName              => 'myshop.de',
    SecondaryDomains        => ['yourshop.de'],
    HasSSLCertificate       => 1,
    WebServerScriptNamePart => 'shop',
};

sub Main {
    # temporary change settings
    my $EpagesConf = EpagesConfig();
    $EpagesConf->set('DE_EPAGES::Object', 'IsWebServerScriptNamePartUnique', 0);
    $EpagesConf->set('DE_EPAGES::Object', 'ExpandWWWDomain', 0);
    ok ( !ISWEBSERVERSCRIPTNAMEPARTUNIQUE, "not IsWebServerScriptNamePartUnique" );
    ok ( !EXPANDWWWDOMAIN, "not ExpandWWWDomain" );

    subtest SSLProxy => sub {
        plan tests => 48;
        local $ENV{'EPAGES_SSL_PROXY'} = 'sslproxy.com'; # :4443, '/proxy'

        note 'shop without SSL certificate';
        my $hInfo1 = Urls($hSite1, $hSystem);
        is ($hInfo1->{'ProtocolAndServer'}, 'http://www.myshop.de:40', 'ProtocolAndServer');
        is ($hInfo1->{'ProtocolAndServerSSL'}, 'https://sslproxy.com/www.myshop.de%3A40', 'ProtocolAndServerSSL');
        is ($hInfo1->{'Server'}, 'www.myshop.de:40', 'Server');
        is ($hInfo1->{'ServerSSL'}, 'sslproxy.com', 'ServerSSL');
        is ($hInfo1->{'WebServerScriptAdminUrl'}, 'http://www.myshop.de:40/epages/shop.admin', 'WebServerScriptAdminUrl');
        is ($hInfo1->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin');
        is ($hInfo1->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL');
        is ($hInfo1->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront');
        is ($hInfo1->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL');
        is ($hInfo1->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension');
        is ($hInfo1->{'WebServerScriptStorefrontUrl'}, 'http://www.myshop.de:40/epages/shop.sf', 'WebServerScriptStorefrontUrl');
        is ($hInfo1->{'WebServerScriptStorefrontUrlSSL'}, 'https://sslproxy.com/www.myshop.de%3A40/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL');
        is ($hInfo1->{'WebServerScriptAdminSSLUrl'}, 'https://sslproxy.com/www.myshop.de%3A40/epages/shop.admin', 'WebServerScriptAdminSSLUrl');

        note 'shop without domain and without SSL certificate';
        my $hInfo2 = Urls($hSite2, $hSystem);
        is ($hInfo2->{'ProtocolAndServer'}, 'http://www.epages.de:40', 'ProtocolAndServer');
        is ($hInfo2->{'ProtocolAndServerSSL'}, 'https://sslproxy.com/www.epages.de%3A40', 'ProtocolAndServerSSL');
        is ($hInfo2->{'Server'}, 'www.epages.de:40', 'Server');
        is ($hInfo2->{'ServerSSL'}, 'sslproxy.com', 'ServerSSL');
        is ($hInfo2->{'WebServerScriptAdminUrl'}, 'http://www.epages.de:40/epages/myshop.admin', 'WebServerScriptAdminUrl');
        is ($hInfo2->{'WebServerScriptNameAdmin'}, '/epages/myshop.admin', 'WebServerScriptNameAdmin');
        is ($hInfo2->{'WebServerScriptNameAdminSSL'}, '/epages/myshop.admin', 'WebServerScriptNameAdminSSL');
        is ($hInfo2->{'WebServerScriptNameStorefront'}, '/epages/myshop.sf', 'WebServerScriptNameStorefront');
        is ($hInfo2->{'WebServerScriptNameStorefrontSSL'}, '/epages/myshop.sf', 'WebServerScriptNameStorefrontSSL');
        is ($hInfo2->{'WebServerScriptNameWithOutExtension'}, '/epages/myshop.', 'WebServerScriptNameWithOutExtension');
        is ($hInfo2->{'WebServerScriptStorefrontUrl'}, 'http://www.epages.de:40/epages/myshop.sf', 'WebServerScriptStorefrontUrl');
        is ($hInfo2->{'WebServerScriptStorefrontUrlSSL'}, 'https://sslproxy.com/www.epages.de%3A40/epages/myshop.sf', 'WebServerScriptStorefrontUrlSSL');
        is ($hInfo2->{'WebServerScriptAdminSSLUrl'}, 'https://sslproxy.com/www.epages.de%3A40/epages/myshop.admin', 'WebServerScriptAdminSSLUrl');

        note 'shop with SSL certificate';
        my $hInfo3 = Urls($hSite3, $hSystem);
        is ($hInfo3->{'ProtocolAndServer'}, 'http://1.2.3.4:40', 'ProtocolAndServer');
        is ($hInfo3->{'ProtocolAndServerSSL'}, 'https://1.2.3.4', 'ProtocolAndServerSSL');
        is ($hInfo3->{'Server'}, '1.2.3.4:40', 'Server');
        is ($hInfo3->{'ServerSSL'}, '1.2.3.4', 'ServerSSL');
        is ($hInfo3->{'WebServerScriptAdminUrl'}, 'http://1.2.3.4:40/epages/shop.admin', 'WebServerScriptAdminUrl');
        is ($hInfo3->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin');
        is ($hInfo3->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL');
        is ($hInfo3->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront');
        is ($hInfo3->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL');
        is ($hInfo3->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension');
        is ($hInfo3->{'WebServerScriptStorefrontUrl'}, 'http://1.2.3.4:40/epages/shop.sf', 'WebServerScriptStorefrontUrl');
        is ($hInfo3->{'WebServerScriptStorefrontUrlSSL'}, 'https://1.2.3.4/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL');

        note 'SSL proxy with port number';
        local $ENV{'EPAGES_SSL_PROXY'} = 'sslproxy.com:4443';
        my $hInfo5 = Urls($hSite1, $hSystem);
        is ($hInfo5->{'ProtocolAndServer'}, 'http://www.myshop.de:40', 'ProtocolAndServer');
        is ($hInfo5->{'ProtocolAndServerSSL'}, 'https://sslproxy.com:4443/www.myshop.de%3A40', 'ProtocolAndServerSSL');
        is ($hInfo5->{'Server'}, 'www.myshop.de:40', 'Server');
        is ($hInfo5->{'ServerSSL'}, 'sslproxy.com:4443', 'ServerSSL');
        is ($hInfo5->{'WebServerScriptAdminSSLUrl'}, 'https://sslproxy.com:4443/www.myshop.de%3A40/epages/shop.admin', 'WebServerScriptAdminSSLUrl');

        note 'SSL proxy with path';
        local $ENV{'EPAGES_SSL_PROXY'} = 'sslproxy.com/proxy';

        my $hInfo6 = Urls($hSite1, $hSystem);
        is ($hInfo6->{'ProtocolAndServer'}, 'http://www.myshop.de:40', 'ProtocolAndServer');
        is ($hInfo6->{'ProtocolAndServerSSL'}, 'https://sslproxy.com/proxy/www.myshop.de%3A40', 'ProtocolAndServerSSL');
        is ($hInfo6->{'Server'}, 'www.myshop.de:40', 'Server');
        is ($hInfo6->{'ServerSSL'}, 'sslproxy.com/proxy', 'ServerSSL');
        is ($hInfo6->{'WebServerScriptAdminSSLUrl'}, 'https://sslproxy.com/proxy/www.myshop.de%3A40/epages/shop.admin', 'WebServerScriptAdminSSLUrl');
    };

    my $hInfo1 = Urls($hSite1, $hSystem);
    is ($hInfo1->{'ProtocolAndServer'}, 'http://www.myshop.de:40', 'ProtocolAndServer');
    is ($hInfo1->{'ProtocolAndServerSSL'}, 'https://www.epages.de', 'ProtocolAndServerSSL');
    is ($hInfo1->{'Server'}, 'www.myshop.de:40', 'Server');
    is ($hInfo1->{'ServerSSL'}, 'www.epages.de', 'ServerSSL');
    is ($hInfo1->{'WebServerScriptAdminUrl'}, 'http://www.myshop.de:40/epages/shop.admin', 'WebServerScriptAdminUrl');
    is ($hInfo1->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin');
    is ($hInfo1->{'WebServerScriptNameAdminSSL'}, '/epages/Store.admin', 'WebServerScriptNameAdminSSL');
    is ($hInfo1->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront');
    is ($hInfo1->{'WebServerScriptNameStorefrontSSL'}, '/epages/Store.sf', 'WebServerScriptNameStorefrontSSL');
    is ($hInfo1->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension');
    is ($hInfo1->{'WebServerScriptStorefrontUrl'}, 'http://www.myshop.de:40/epages/shop.sf', 'WebServerScriptStorefrontUrl');
    is ($hInfo1->{'WebServerScriptStorefrontUrlSSL'}, 'https://www.epages.de/epages/Store.sf', 'WebServerScriptStorefrontUrlSSL');
    is ($hInfo1->{'WebServerScriptAdminSSLUrl'}, 'https://www.epages.de/epages/Store.admin', 'WebServerScriptAdminSSLUrl');

    foreach my $HasSSLCertificate (0,1) {
        my $hInfo2 = Urls({ %$hSite2, HasSSLCertificate => $HasSSLCertificate}, $hSystem);
        is ($hInfo2->{'ProtocolAndServer'}, 'http://www.epages.de:40', 'ProtocolAndServer');
        is ($hInfo2->{'ProtocolAndServerSSL'}, 'https://www.epages.de', 'ProtocolAndServerSSL');
        is ($hInfo2->{'Server'}, 'www.epages.de:40', 'Server');
        is ($hInfo2->{'ServerSSL'}, 'www.epages.de', 'ServerSSL');
        is ($hInfo2->{'WebServerScriptAdminUrl'}, 'http://www.epages.de:40/epages/myshop.admin', 'WebServerScriptAdminUrl');
        is ($hInfo2->{'WebServerScriptNameAdmin'}, '/epages/myshop.admin', 'WebServerScriptNameAdmin');
        is ($hInfo2->{'WebServerScriptNameAdminSSL'}, '/epages/myshop.admin', 'WebServerScriptNameAdminSSL');
        is ($hInfo2->{'WebServerScriptNameStorefront'}, '/epages/myshop.sf', 'WebServerScriptNameStorefront');
        is ($hInfo2->{'WebServerScriptNameStorefrontSSL'}, '/epages/myshop.sf', 'WebServerScriptNameStorefrontSSL');
        is ($hInfo2->{'WebServerScriptNameWithOutExtension'}, '/epages/myshop.', 'WebServerScriptNameWithOutExtension');
        is ($hInfo2->{'WebServerScriptStorefrontUrl'}, 'http://www.epages.de:40/epages/myshop.sf', 'WebServerScriptStorefrontUrl');
        is ($hInfo2->{'WebServerScriptStorefrontUrlSSL'}, 'https://www.epages.de/epages/myshop.sf', 'WebServerScriptStorefrontUrlSSL');
    }

    my $hInfo3 = Urls($hSite3, $hSystem);
    is ($hInfo3->{'ProtocolAndServer'}, 'http://1.2.3.4:40', 'ProtocolAndServer');
    is ($hInfo3->{'ProtocolAndServerSSL'}, 'https://1.2.3.4', 'ProtocolAndServerSSL');
    is ($hInfo3->{'Server'}, '1.2.3.4:40', 'Server');
    is ($hInfo3->{'ServerSSL'}, '1.2.3.4', 'ServerSSL');
    is ($hInfo3->{'WebServerScriptAdminUrl'}, 'http://1.2.3.4:40/epages/shop.admin', 'WebServerScriptAdminUrl');
    is ($hInfo3->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin');
    is ($hInfo3->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL');
    is ($hInfo3->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront');
    is ($hInfo3->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL');
    is ($hInfo3->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension');
    is ($hInfo3->{'WebServerScriptStorefrontUrl'}, 'http://1.2.3.4:40/epages/shop.sf', 'WebServerScriptStorefrontUrl');
    is ($hInfo3->{'WebServerScriptStorefrontUrlSSL'}, 'https://1.2.3.4/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL');

    $hInfo3 = Urls($hSite4, $hSystem);
    is ($hInfo3->{'ProtocolAndServer'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40', 'ProtocolAndServer');
    is ($hInfo3->{'ProtocolAndServerSSL'}, 'https://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]', 'ProtocolAndServerSSL');
    is ($hInfo3->{'Server'}, '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40', 'Server');
    is ($hInfo3->{'ServerSSL'}, '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]', 'ServerSSL');
    is ($hInfo3->{'WebServerScriptAdminUrl'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40/epages/shop.admin', 'WebServerScriptAdminUrl');
    is ($hInfo3->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin');
    is ($hInfo3->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL');
    is ($hInfo3->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront');
    is ($hInfo3->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL');
    is ($hInfo3->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension');
    is ($hInfo3->{'WebServerScriptStorefrontUrl'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40/epages/shop.sf', 'WebServerScriptStorefrontUrl');
    is ($hInfo3->{'WebServerScriptStorefrontUrlSSL'}, 'https://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL');

    my $hInfo4 = Urls($hSite1, $hSystem2);
    is ($hInfo4->{'ProtocolAndServerSSL'}, 'http://www.myshop.de:40', 'ProtocolAndServerSSL without ssl');

    foreach my $HasSSLCertificate (0,1) {
        my $hInfo2 = Urls({ %$hSite2, HasSSLCertificate => $HasSSLCertificate}, $hSystem2);
        is ($hInfo2->{'ProtocolAndServer'}, 'http://www.epages.de:40', 'ProtocolAndServer');
        is ($hInfo2->{'ProtocolAndServerSSL'}, 'http://www.epages.de:40', 'ProtocolAndServerSSL');
        is ($hInfo2->{'Server'}, 'www.epages.de:40', 'Server');
        is ($hInfo2->{'ServerSSL'}, undef, 'ServerSSL');
        is ($hInfo2->{'WebServerScriptAdminUrl'}, 'http://www.epages.de:40/epages/myshop.admin', 'WebServerScriptAdminUrl');
        is ($hInfo2->{'WebServerScriptNameAdmin'}, '/epages/myshop.admin', 'WebServerScriptNameAdmin');
        is ($hInfo2->{'WebServerScriptNameAdminSSL'}, '/epages/myshop.admin', 'WebServerScriptNameAdminSSL');
        is ($hInfo2->{'WebServerScriptNameStorefront'}, '/epages/myshop.sf', 'WebServerScriptNameStorefront');
        is ($hInfo2->{'WebServerScriptNameStorefrontSSL'}, '/epages/myshop.sf', 'WebServerScriptNameStorefrontSSL');
        is ($hInfo2->{'WebServerScriptNameWithOutExtension'}, '/epages/myshop.', 'WebServerScriptNameWithOutExtension');
        is ($hInfo2->{'WebServerScriptStorefrontUrl'}, 'http://www.epages.de:40/epages/myshop.sf', 'WebServerScriptStorefrontUrl');
        is ($hInfo2->{'WebServerScriptStorefrontUrlSSL'}, 'http://www.epages.de:40/epages/myshop.sf', 'WebServerScriptStorefrontUrlSSL');
    }

    my $hInfo5 = Urls($hSite3, $hSystem2);
    is ($hInfo5->{'ProtocolAndServer'}, 'http://1.2.3.4:40', 'ProtocolAndServer with ssl on site');
    is ($hInfo5->{'ProtocolAndServerSSL'}, 'https://1.2.3.4', 'ProtocolAndServerSSL with ssl on site');
    is ($hInfo5->{'Server'}, '1.2.3.4:40', 'Server with ssl on site');
    is ($hInfo5->{'ServerSSL'}, '1.2.3.4', 'ServerSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptAdminUrl'}, 'http://1.2.3.4:40/epages/shop.admin', 'WebServerScriptAdminUrl with ssl on site');
    is ($hInfo5->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin with ssl on site');
    is ($hInfo5->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront with ssl on site');
    is ($hInfo5->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension with ssl on site');
    is ($hInfo5->{'WebServerScriptStorefrontUrl'}, 'http://1.2.3.4:40/epages/shop.sf', 'WebServerScriptStorefrontUrl with ssl on site');
    is ($hInfo5->{'WebServerScriptStorefrontUrlSSL'}, 'https://1.2.3.4/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL with ssl on site');

    $hInfo5 = Urls($hSite4, $hSystem2);
    is ($hInfo5->{'ProtocolAndServerSSL'}, 'https://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]', 'ProtocolAndServerSSL with ssl on site');
    is ($hInfo5->{'ProtocolAndServer'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40', 'ProtocolAndServer with ssl on site');
    is ($hInfo5->{'ProtocolAndServerSSL'}, 'https://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]', 'ProtocolAndServerSSL with ssl on site');
    is ($hInfo5->{'Server'}, '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40', 'Server with ssl on site');
    is ($hInfo5->{'ServerSSL'}, '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]', 'ServerSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptAdminUrl'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40/epages/shop.admin', 'WebServerScriptAdminUrl with ssl on site');
    is ($hInfo5->{'WebServerScriptNameAdmin'}, '/epages/shop.admin', 'WebServerScriptNameAdmin with ssl on site');
    is ($hInfo5->{'WebServerScriptNameAdminSSL'}, '/epages/shop.admin', 'WebServerScriptNameAdminSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptNameStorefront'}, '/epages/shop.sf', 'WebServerScriptNameStorefront with ssl on site');
    is ($hInfo5->{'WebServerScriptNameStorefrontSSL'}, '/epages/shop.sf', 'WebServerScriptNameStorefrontSSL with ssl on site');
    is ($hInfo5->{'WebServerScriptNameWithOutExtension'}, '/epages/shop.', 'WebServerScriptNameWithOutExtension with ssl on site');
    is ($hInfo5->{'WebServerScriptStorefrontUrl'}, 'http://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40/epages/shop.sf', 'WebServerScriptStorefrontUrl with ssl on site');
    is ($hInfo5->{'WebServerScriptStorefrontUrlSSL'}, 'https://[2001:4bd8:2:2:213:d3ff:fe62:f1f0]/epages/shop.sf', 'WebServerScriptStorefrontUrlSSL with ssl on site');

    ConnectTestSite(); # only for Hook PoolDBEntries

    # SSL enabled, system entries
    ComparePoolDBEntries( 'PooolDBEntries system',
        PoolDBEntries($hSystem, $hSystem), [
            { 'URI' => '/epages/Store.', 'Host' => 'www.epages.de'  },
            { 'URI' => '/epages/Store.', 'Host' => 'www.epages.de:40' },
        ] );

    # SSL enabled, shop with short URL and domain name, but without SSL cert
    ComparePoolDBEntries( 'PooolDBEntries 1',
        PoolDBEntries($hSite1, $hSystem), [
            { 'URI' => '/epages/shop.',  'Host' => 'www.myshop.de:40' },
        ] );

    # SSL enabled, shop with short URL but without domain name
    ComparePoolDBEntries( 'PooolDBEntries 2',
        PoolDBEntries($hSite2, $hSystem), [
            { 'URI' => '/epages/myshop.', 'Host' => 'www.epages.de'  },
            { 'URI' => '/epages/myshop.', 'Host' => 'www.epages.de:40' },
        ] );

    # SSL enabled, shop with short URL and domain name and SSL cert
    ComparePoolDBEntries( 'PooolDBEntries 3',
        PoolDBEntries($hSite3, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => '1.2.3.4'  },
            { 'URI' => '/epages/shop.', 'Host' => '1.2.3.4:40' },
        ] );

    # SSL enabled, shop with short URL and domain name and SSL cert
    ComparePoolDBEntries( 'PooolDBEntries 3',
        PoolDBEntries($hSite4, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]' },
            { 'URI' => '/epages/shop.', 'Host' => '[2001:4bd8:2:2:213:d3ff:fe62:f1f0]:40' },
        ] );

    # SSL enabled, shop without short URL and without domain name
    ComparePoolDBEntries( 'PooolDBEntries 4',
        PoolDBEntries($hSite5, $hSystem), [ ] );

    $EpagesConf->set('DE_EPAGES::Object', 'IsWebServerScriptNamePartUnique', 1);
    ok ( ISWEBSERVERSCRIPTNAMEPARTUNIQUE, "IsWebServerScriptNamePartUnique" );

    # SSL enabled, shop with short URL and domain name, but without SSL cert
    ComparePoolDBEntries( 'PooolDBEntries 5',
        PoolDBEntries($hSite1, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de'},
        ] );

    # SSL enabled, shop with short URL, domain name and additional domain, but without SSL cert
    ComparePoolDBEntries( 'PoolDBEntries 6',
        PoolDBEntries($hSite6, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de:40' },
        ]
    );

    # SSL enabled, shop with short URL, domain name, additional domain and SSL cert
    ComparePoolDBEntries( 'PoolDBEntries 7',
        PoolDBEntries($hSite7, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de:40' },
        ]
    );

    $EpagesConf->set('DE_EPAGES::Object', 'ExpandWWWDomain', 1);
    ok ( EXPANDWWWDOMAIN, "ExpandWWWDomain" );

    ComparePoolDBEntries( 'PooolDBEntries 8',
        PoolDBEntries($hSite1, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.epages.de' },
        ] );

    $EpagesConf->set('DE_EPAGES::Object', 'IsWebServerScriptNamePartUnique', 0);
    ok ( !ISWEBSERVERSCRIPTNAMEPARTUNIQUE, "not IsWebServerScriptNamePartUnique" );

    ComparePoolDBEntries( 'PooolDBEntries 9',
        PoolDBEntries($hSite1, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
        ] );

    ComparePoolDBEntries( 'PoolDBEntries 10',
        PoolDBEntries($hSite6, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.yourshop.de:40' },
        ]
    );

    ComparePoolDBEntries( 'PoolDBEntries 11',
        PoolDBEntries($hSite7, $hSystem), [
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'yourshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.yourshop.de' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.myshop.de:40' },
            { 'URI' => '/epages/shop.', 'Host' => 'www.yourshop.de:40' },
        ]
    );

    ok( TestWebServerScriptNamePart('09azAZ:@&=+$,_-'), 'TestWebServerScriptNamePart 1' );
    ok( !TestWebServerScriptNamePart('meinshop.de'), 'TestWebServerScriptNamePart 2' );
    ok( TestWebServerScriptNamePart('mein-shop'), 'TestWebServerScriptNamePart 3' );
    ok( TestWebServerScriptNamePart('mein_shop'), 'TestWebServerScriptNamePart 4' );
    ok( !TestWebServerScriptNamePart('shöppchen'), 'TestWebServerScriptNamePart 5' );
    ok( !TestWebServerScriptNamePart('meinshop '), 'TestWebServerScriptNamePart 6' );

    DisconnectSite();
}

sub ComparePoolDBEntries($$$) {
    my( $TestName, $aEntries, $aRefEntries ) = @_;

    my %Entries = map { $_->{Host} => { $_->{URI} => undef } } @$aEntries;
    is ( scalar @$aEntries, scalar @$aRefEntries, "$TestName: count" );
    foreach my $hRefEntry (@$aRefEntries) {
        my $Host = $hRefEntry->{Host};
        my $Uri = $hRefEntry->{URI};
        ok ( exists $Entries{$Host}->{$Uri}, "$TestName: entry $Host$Uri" );
    }
}

RunScript(Sub=>\&Main, Debug=>1);
