use Test::More tests => 94;
use Test::Deep;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Product qw (
    ExistsSubProductByAttributeValues
    GetSubProductByAttributeValues
    SmartSearchProducts
    UsedVariationValues
);

## no critic qw( ProhibitMagicNumbers )

my $ROOTPATH = '/Test';

sub TestBundle {

    my $Product = LoadObjectByPath('/Test/TestShop/Products/Bundle');
    ok( $Product->get('IsBundleProduct'), 'Product.IsBundleProduct' );
    ok( !$Product->get('IsDownloadProduct'), 'not Product.IsDownloadProduct' );
    ok( !$Product->get('IsAppointment'), 'not Product.IsAppointment' );

    my $aBundledProducts = $Product->get('BundledProducts');
    is( scalar @$aBundledProducts, 2, 'Product.BundledProducts' );

    my $Product1 = $aBundledProducts->[0];
    my $aIncludedInBundles = $Product1->get('IncludedInBundles');
    is( scalar @$aIncludedInBundles, 1, 'Product.IncludedInBundles' );
    ok( $aIncludedInBundles->[0]->id == $Product->id, 'Product.IncludedInBundles' );
}

sub TestDownload {

    my $Product = LoadObjectByPath('/Test/TestShop/Products/DownloadProduct');
    ok( $Product->get('IsDownloadProduct'), 'Product.IsDownloadProduct' );
    ok( !$Product->get('IsAppointment'), 'not Product.IsAppointment' );
    ok( !$Product->get('IsBundleProduct'), 'not Product.IsBundleProduct' );

    my $aDownloads = $Product->get('DownloadProductMaps');
    is( scalar @$aDownloads, 1, 'Product.DownloadProductMaps' );

    my $aDirectDownloads = $Product->get('DirectDownloadProductMaps');
    is( scalar @$aDownloads, 1, 'Product.DirectDownloadProductMaps' );

    my $ProductEN = $Product->child('SubProducts')->child('DownloadProduct-en');
    $aDownloads = $ProductEN->get('DownloadProductMaps');
    is( scalar @$aDownloads, 2, 'SubProducts.DownloadProductMaps' );
    is( $aDownloads->[0]->get('TargetUrl'), 'Product.exe', 'files of master product before sub product' );
    is( $aDownloads->[1]->get('TargetUrl'), 'Product-en.dll', 'files of sub product after master product' );

    $aDirectDownloads = $ProductEN->get('DirectDownloadProductMaps');
    is( scalar @$aDirectDownloads, 1, 'SubProduct.DirectDownloadProductMaps' );

    my $ProductDE = $Product->child('SubProducts')->child('DownloadProduct-de');
    $aDownloads = $ProductDE->get('DownloadProductMaps');
    is( scalar @$aDownloads, 4, 'DownloadProductMaps of DownloadProduct-de' );
    my $DownloadLocal = $aDownloads->[0];
    is( $DownloadLocal->get('IsExternal'), 0, 'DownloadProductMap.IsExternal' );
    is( $DownloadLocal->get('FileName'), 'Product.exe', 'DownloadProductMap.FileName for local file' );

    my $DownloadExtern = $aDownloads->[3];
    is( $DownloadExtern->get('IsExternal'), 1, 'DownloadProductMap.IsExternal' );
    is( $DownloadExtern->get('FileName'), 'epages7.exe', 'DownloadProductMap.FileName for external file' );
}

sub TestAppointment {

    my $Product = LoadObjectByPath('/Test/TestShop/Appointments/Table4Person');
    ok( $Product->get('IsAppointment'), 'Product.IsAppointment' );
    ok( !$Product->get('IsDownloadProduct'), 'not Product.IsDownloadProduct' );
    ok( !$Product->get('IsBundleProduct'), 'not Product.IsBundleProduct' );

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    my $StartDate = $Formatter->parse_datetime('20.02.2008 16:20:00','datetime', 'medium');
    my $EndDate1 = $Formatter->parse_datetime('20.02.2009 16:13:30','datetime', 'medium');
    my $EndDate2 = $Formatter->parse_datetime('20.02.2009 16:23:30','datetime', 'medium');
    my $EndDate3 = $Formatter->parse_datetime('20.02.2010 16:23:30','datetime', 'medium');
    my $EndDate4 = $Formatter->parse_datetime('20.02.2008 16:50:00','datetime', 'medium');
    my $EndDate5 = $Formatter->parse_datetime('20.02.2008 17:20:00','datetime', 'medium');
    my $EndDate6 = $Formatter->parse_datetime('20.02.2008 18:20:00','datetime', 'medium');

    is ($Product->timeFactor($StartDate,$EndDate1), 4391.945833, '2 hours without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 4392.029167, '2 hours with overlap');
    is ($Product->timeFactor($StartDate,$EndDate4), 0.25, '1/2 hour');
    is ($Product->timeFactor($StartDate,$EndDate5), 0.5, '1 hour');
    is ($Product->timeFactor($StartDate,$EndDate6), 1, '2 hours');

    $Product->set({DurationUnit=>LoadObjectByPath('/Units/second/month'), DurationAmount=>1});

    is ($Product->timeFactor($StartDate,$EndDate1), 11.966667, 'month without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 12.033333, 'month with overlap');

    $Product->set({DurationUnit=>LoadObjectByPath('/Units/second/month'), DurationAmount=>2});
    is ($Product->timeFactor($StartDate,$EndDate1), 5.983333, '2 month without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 6.016667, '2 month with overlap');

    $Product->set({DurationUnit=>LoadObjectByPath('/Units/second/year'), DurationAmount=>1});
    is ($Product->timeFactor($StartDate,$EndDate1), 0.997222, 'year without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 1.002778, 'year with overlap');
    is ($Product->timeFactor($StartDate,$EndDate3), 2.002778, 'year with more overlap');
    is ($Product->timeFactor($StartDate,$EndDate5), 0.002778, '1 hour for duration of 1 year');

    $Product->set({DurationUnit=>LoadObjectByPath('/Units/second/year'), DurationAmount=>2});
    is ($Product->timeFactor($StartDate,$EndDate1), 0.498611, '2 year without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 0.501389, '2 years with overlap');
    is ($Product->timeFactor($StartDate,$EndDate3), 1.001389, '2 years with more overlap');
}

sub TestSmartSearch {
    my $Shop = LoadObjectByPath('/Test/TestShop');

    my $aProducts = SmartSearchProducts($Shop, 'SmartSearch-');

    is (scalar(@$aProducts), 1, 'Smartsearch founds one');
    is ($aProducts->[0]->get('Alias'), 'SmartSearch-123', 'SmartSearch finds the right product');

    $aProducts = SmartSearchProducts($Shop, 'SmartSearch_');

    is (scalar(@$aProducts), 1, 'Smartsearch founds one');
    is ($aProducts->[0]->get('Alias'), 'SmartSearch_123', 'SmartSearch finds the right product');

    $aProducts = SmartSearchProducts($Shop, 'SmartSearch');
    is (scalar(@$aProducts), 2, 'Smartsearch founds one');
}

sub TestGetSubProductByAttributeValues {
    # check erroneously passed parameters
    eval {
        GetSubProductByAttributeValues(undef, undef);
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_MISSING', 'missing parameters (2x undef) (GetSubProductByAttributeValues)' );

    eval {
        GetSubProductByAttributeValues('String', undef);
    };
    ok( ExistsError() && GetError()->code eq 'FORMAT_NOT_INTEGER', 'first parameter is no integer (GetSubProductByAttributeValues)' );

    eval {
        GetSubProductByAttributeValues(12345, 67890);
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_NO_ARRAY', 'second parameter no array (GetSubProductByAttributeValues)' );

    # define test cases
    my $haExpectedResults = {
        'MyJacket' => [
            {
                'SubProduct' => 'JacketRed1',
                'Color'      => 'ExtremeRed',
                'Size'       => 'S',
            },
            {
                'SubProduct' => 'JacketBlack3',
                'Color'      => 'Black',
                'Size'       => 'L',
            }
        ],
    };

    # validate the correctness of GetSubProductByAttributeValues based on the given test shop
    foreach my $ProductAlias ( %{ $haExpectedResults } ) {
        next if ref $ProductAlias eq 'ARRAY'; #fixme
        my $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/')->child($ProductAlias);
        my $ProductType = LoadObjectByPath('/Test/TestShop/ProductTypes/')->child($SuperProduct->class->alias);

        # match localized variation values with their ids in the database
        my %hProductAttributes;
        foreach my $ProductAttribute ( @{ $ProductType->directAttributes } ) {
            my $ahPreDefAttributes = $ProductAttribute->get('PreDefAttributes');
            foreach my $hPreDefAttribute ( @{ $ahPreDefAttributes } ) {
                $hProductAttributes{$ProductAttribute->alias}{$hPreDefAttribute->alias} = $hPreDefAttribute->id;
            }
        }

        foreach my $hEntry ( @{ $haExpectedResults->{$ProductAlias} } ) {
            my @aVariationString;
            push @aVariationString, $hProductAttributes{'Color'}{$hEntry->{'Color'}} if exists $hEntry->{'Color'};
            push @aVariationString, $hProductAttributes{'Size'}{$hEntry->{'Size'}}   if exists $hEntry->{'Size'};
            my $Exists = ExistsSubProductByAttributeValues($SuperProduct->id, \@aVariationString);
            ok( $Exists, 'sub product having the combination of variations exists' );
            my $SubProduct = GetSubProductByAttributeValues($SuperProduct->id, \@aVariationString);
            is( $SubProduct->alias, $hEntry->{'SubProduct'}, 'verify the sub products found for variations ' . join(',', @aVariationString) );
        }

        # error expected: invalid attribute id
        my $Exists = ExistsSubProductByAttributeValues($SuperProduct->id, [ qw( 12345 ) ]);
        ok( !$Exists, 'given variation does not exist' );

        eval {
            GetSubProductByAttributeValues($SuperProduct->id, [ qw( 12345 ) ]);
        };
        ok( ExistsError() && GetError()->code eq 'SUBPRODUCT_NOT_EXISTS', 'error: variation does not exist' );

        # error expected: too many sub products are matching
        eval {
            my @aVariationString;
            push @aVariationString, $hProductAttributes{'Color'}{'ExtremeRed'};
            GetSubProductByAttributeValues($SuperProduct->id, \@aVariationString);
        };
        ok( ExistsError() && GetError()->code eq 'TOO_MANY_MATCHING_SUBPRODUCTS', 'error: more than one subproduct found' );
    }

    return;
}

sub TestUsedVariationValues {
    # check erroneously passed parameters
    eval {
        UsedVariationValues();
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_MISSING', 'missing parameter (UsedVariationValues)' );

    eval {
        UsedVariationValues('String');
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_NO_OBJECT', 'wrong parameter passed (UsedVariationValues)' );

    eval {
        UsedVariationValues(undef);
    };
    ok( ExistsError() && GetError()->code eq 'PARAMETER_MISSING', 'undefined parameter passed (UsedVariationValues)' );

    # validate the correctness of UsedVariationValues based on the given test shop
    my $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/MyJacket');
    CompareUsedVariationValues($SuperProduct, [ qw( ExtremeRed Black S M L XL XXL ) ] );

    $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/DownloadProduct');
    CompareUsedVariationValues($SuperProduct, [ qw( Deutsch Englisch French ) ] );

    $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/MyPetrol');
    CompareUsedVariationValues($SuperProduct, [ () ] );

    return;
}

sub TestDisplayTitle {

    my $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/MyJacket');
    my $SubProduct = $SuperProduct->child('SubProducts')->child('JacketBlack5');
    my $EN = GetPKeyLanguageByCode('en');
    my $Shop = $SuperProduct->getSite;
    my $OldShopName = $Shop->get('Name', $EN);

    $Shop->set({'Name' => 'Shop Name'}, $EN);

    # product without name and title
    is( $SuperProduct->getWithParent('Title', $EN), undef, 'Initial SuperProduct Title' );
    is( $SuperProduct->getWithParent('DisplayTitle', $EN), 'MyJacket - Shop Name', 'Initial SuperProduct DisplayTitle' );
    is( $SubProduct->getWithParent('Title', $EN), undef, 'Initial SubProduct Title' );
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'JacketBlack5 - Shop Name', 'Initial SubProduct DisplayTitle' );

    # super product with name, but no title
    $SuperProduct->set({'Name' => 'New SuperProduct Name'}, $EN);
    is( $SuperProduct->getWithParent('Title', $EN), undef, 'SuperProduct.Title (New SuperProduct Name)' );
    is( $SuperProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Name - Shop Name', 'SuperProduct.DisplayTitle (New SuperProduct Name)' );
    is( $SubProduct->getWithParent('Title', $EN), undef, 'SubProduct.Title (New SuperProduct Name)' );
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Name - Shop Name', 'SubProduct.DisplayTitle (New SuperProduct Name)' );

    # super product with title, sub product without title
    $SuperProduct->set({'Title' => 'New SuperProduct Title'}, $EN);
    is( $SuperProduct->getWithParent('Title', $EN), 'New SuperProduct Title', 'SuperProduct.Title (New SuperProduct Title)' );
    is( $SuperProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Title', 'SuperProduct.DisplayTitle (New SuperProduct Title)' );
    is( $SubProduct->getWithParent('Title', $EN), 'New SuperProduct Title', 'SubProduct.Title (New SuperProduct Title)' );
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Title', 'SubProduct.DisplayTitle (New SuperProduct Title)' );

    # sub product with title
    $SubProduct->set({'Title' => 'New SubProduct Title'}, $EN);
    is( $SuperProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Title', 'SuperProduct.DisplayTitle (New SubProduct Title)' );
    is( $SubProduct->getWithParent('Title', $EN), 'New SubProduct Title', 'SubProduct.Title (New SuperProduct Title)' );
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'New SubProduct Title', 'SubProduct.DisplayTitle (New SuperProduct Title)' );

    # product name = shop name
    $SuperProduct->set({'Title' => undef}, $EN);
    $SubProduct->set({'Title' => undef, 'Name' => 'Shop Name'}, $EN);
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'Shop Name', 'Product Name = Shop Name' );

    # undefined shop name
    $Shop->set({'Name' => undef}, $EN);
    $SubProduct->set({'Name' => 'Sub Product Name'}, $EN);
    is( $SubProduct->getWithParent('DisplayTitle', $EN), 'Sub Product Name', 'Undefined Shop Name' );
    is( $SuperProduct->getWithParent('DisplayTitle', $EN), 'New SuperProduct Name', 'Undefined Shop Name' );

    # cleanup
    $Shop->set({'Name' => $OldShopName}, $EN);
    $SubProduct->set({'Title' => undef}, $EN);
    $SuperProduct->set({'Title' => undef}, $EN);
}

sub CompareUsedVariationValues {
    my ($SuperProduct, $aExpectedVariationValues) = @_;

    my $aGotPreDefAttributeIDs = UsedVariationValues($SuperProduct);
    my @aGotVariationValues;
    push @aGotVariationValues, LoadObject($_)->alias foreach @{ $aGotPreDefAttributeIDs };
    cmp_deeply( \@aGotVariationValues, bag(@{ $aExpectedVariationValues }), 'checked used variations for ' . $SuperProduct->alias );

    return;
}

sub TestReferencePrice {
    my $Product = LoadObjectByPath('/Test/TestShop/Products/ReferencePriceProduct');
    ok( !$Product->get('IsDownloadProduct'), 'not Product.IsDownloadProduct' );
    ok( !$Product->get('IsAppointment'), 'not Product.IsAppointment' );
    ok( !$Product->get('IsBundleProduct'), 'not Product.IsBundleProduct' );

    is ( $Product->getReferencePrice('EUR', 12), 2.4, 'verify reference price');
    is ( $Product->getReferencePrice('EUR', 100, 12), 17.6, 'verify reference price with discount');
    is ( $Product->getReferencePrice('EUR'), undef, 'verify reference price function');

    $Product = LoadObjectByPath('/Test/TestShop/Products/MyJacket');
    $Product->set({ RefAmount => undef , RefContentAmount => 10 });
    is $Product->getReferencePrice('EUR', 100), undef, 'no RefAmount';
    $Product->set({ RefAmount => 10 , RefContentAmount => undef });
    is $Product->getReferencePrice('EUR', 100), undef, 'no RefContentAmount';
    $Product->set({ RefAmount => 1 , RefContentAmount => 1 });
    is $Product->getReferencePrice('EUR', 100.123), 100.12, 'RefAmount == RefContentAmount';
    is $Product->getReferencePrice('BHD', 100.123), 100.123, 'Currency.MinorUnit=3';
    is $Product->getReferencePrice('BIF', 100.123), 100, 'Currency.MinorUnit=0';

    return;
}

sub TestAvailabilityText {

    my $Shop = LoadObjectByPath('/Test/TestShop');
    my $Appointment = $Shop->path('Appointments/Table4Person');
    my $LanguageID = GetPKeyLanguageByCode('en');
    ok(!$Appointment->get('AvailabilityText', $LanguageID), 'AvailabilityText is null for appointments');

    my $Product = $Shop->path('Products/MyPetrol');
    $Product->set({'StockLevel' => 0 });

    is($Product->get('AvailabilityText', $LanguageID), 'StockLevel 0', 'getting availability value for out of stock.');
    is($Product->get('AvailabilityIcon', $LanguageID), 'ProductOutStockIcon', 'getting availability icon for out of stock.');
    return;
}


sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    TestDisplayTitle();
    TestBundle();
    TestDownload();
    TestAppointment();
    TestSmartSearch();
    TestGetSubProductByAttributeValues();
    TestUsedVariationValues();
    TestReferencePrice();
    TestAvailabilityText();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
